from base_st_classes import *

class LocktSuite(NFSSuite):
    """Test operation 13: LOCKT
    """

    def setUp(self):
        NFSSuite.setUp(self)
        self.fh = None
        self.stateid = None
        self.init_connection()

    def tearDown(self):
        self.ncl.close_file(self.fh, self.stateid, error=None)
        try: self.clean_dir(self.tmp_dir)
        except SkipException, e:
            pass

    #
    # Tests that should work
    #
    def testUnlockedFile(self):
        """LOCKT on a regular unlocked file
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        self.ncl.lock_test(self.fh, 0, 0xffffffffffffffff, error=[NFS4_OK])

    def testLockedFile(self):
        """LOCKT on a locked file
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        self.ncl.lock_file(self.fh, self.stateid)
        self.ncl.lock_test(self.fh)

    def testPartialLockedFile1(self):
        """LOCKT on an unlocked portion of a locked file
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        self.ncl.lock_file(self.fh, self.stateid, 100, 50)
        self.ncl.lock_test(self.fh, error=[NFS4_OK])
        
    def testPartialLockedFile2(self):
        """LOCKT on a locked portion of a locked file
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        self.ncl.lock_file(self.fh, self.stateid, 100, 50)
        self.ncl.lock_test(self.fh, 125, 10)
        
    #
    # Tests that may or may not work depending on server
    #
    def test32bitRange(self):
        """LOCKT range over 32 bits should work or return NFS4ERR_BAD_RANGE
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        self.ncl.lock_test(self.fh, len=0xffffffffffff,
                           error=[NFS4_OK, NFS4ERR_BAD_RANGE])

    def testOverlap(self):
        """LOCKT against your own lock should return NFS4_OK or NFS4ERR_LOCK_RANGE
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        self.ncl.lock_file(self.fh, self.stateid, 25, 75)
        self.ncl.lock_test(self.fh, tester="owner",
                           error=[NFS4_OK, NFS4ERR_LOCK_RANGE])

    #
    # Tests for various error conditions
    #
    
    def testZeroLen(self):
        """LOCKT with len=0 should return NFS4ERR_INVAL
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        self.ncl.lock_test(self.fh, len=0, error=[NFS4ERR_INVAL])

    def testLenTooLong(self):
        """LOCKT with offset+len overflow should return NFS4ERR_INVAL
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        self.ncl.lock_test(self.fh, 100, 0xfffffffffffffffe,
                           error=[NFS4ERR_INVAL])

    def testNoCfh(self):
        """LOCKT with no (cfh) should return NFS4ERR_NOFILEHANDLE
        """
        self.fh, self.stateid = self.ncl.create_confirm()

        test_owner = lock_owner4(self.ncl.clientid, "tester")
        lockop = self.ncl.lockt_op(WRITE_LT, 0, 50, test_owner)
        res = self.ncl.do_ops([lockop])
        self.assert_status(res, [NFS4ERR_NOFILEHANDLE])

    def testDirfile(self):
        """LOCKT with a directory should return NFS4ERR_ISDIR
        """
        fh = self.do_rpc(self.ncl.do_getfh, self.dirfile)
        self.ncl.lock_test(fh, error=[NFS4ERR_ISDIR])

    def testNonfile(self):
        """LOCKT with a nonfile object should return NFS4ERR_INVAL
        """
        for path in [self.blockfile,
                     self.charfile,
                     self.linkfile,
                     self.socketfile,
                     self.fifofile]:
            fh = self.do_rpc(self.ncl.do_getfh, path)
            self.ncl.lock_test(fh, error=[NFS4ERR_INVAL])

    def testStaleClientid(self):
        """LOCKT with a bad clientid should return NFS4ERR_STALE_CLIENTID
        """
        self.fh, self.stateid = self.ncl.create_confirm()

        ops = [self.ncl.putfh_op(self.fh)]
        test_owner = lock_owner4(self.get_invalid_clientid(), "tester")
        lockop = self.ncl.lockt_op(WRITE_LT, 0, 50, test_owner)
        ops.append(lockop)
        res = self.ncl.do_ops(ops)
        self.assert_status(res,[NFS4ERR_STALE_CLIENTID])
     
