from base_st_classes import *

class NverifySuite(NFSSuite):
    """Test operation 17: NVERIFY

    FIXME: Add attribute directory and named attribute testing. 

    Equivalence partitioning:

    Input Condition: current filehandle
        Valid equivalence classes:
            link(1)
            block(2)
            char(3)
            socket(4)
            FIFO(5)
            dir(6)
            file(7)
        Invalid equivalence classes:
            invalid filehandle(8)
    Input Condition: obj_attributes.attrmask
        Valid equivalence classes:
            valid attribute(9)
        Invalid equivalence classes:
            invalid attrmask(10) (FATTR4_*_SET)
    Input Condition: obj_attributes.attr_vals
        Valid equivalence classes:
            changed attribute(11)
            same attribute(12)
        Invalid equivalence classes:
            attr with invalid utf8(13)
    """

    def _nverify(self,lookupops,attrdict={FATTR4_SIZE:17},
                error=[NFS4_OK]):
        """call nverify and then getattr
        """
        ops = [self.ncl.putrootfh_op()] + lookupops
        attrs = nfs4lib.dict2fattr(attrdict)
        ops.append(self.ncl.nverify_op(attrs))
        ops.append(self.ncl.getattr(attrdict.keys()))
        res = self.ncl.do_ops(ops)
        self.assert_status(res,error)
        return res

    #
    # Testcases covering valid equivalence classes.
    #
    def testChanged(self):
        """NVERIFY with CHANGED attribute should execute remaining ops

        Covered valid equivalence classes: 1, 2, 3, 4, 5, 6, 7, 9, 11
        """
        # Fetch sizes for all objects
        obj_sizes = self.ncl.lookup_all_objects_and_sizes()
        
        # For each type of object, do nverify with wrong filesize,
        # get new filesize and check if it match previous size. 
        for (lookupops, objsize) in obj_sizes:
            res = self._nverify(lookupops,{FATTR4_SIZE:objsize+17})

            # File sizes should match. 
            obj = res.resarray[-1].arm.arm.obj_attributes
            d =  nfs4lib.fattr2dict(obj)
            new_size = d[FATTR4_SIZE]
            self.failIf(objsize != new_size,
                        "GETATTR after NVERIFY returned different filesize")

    def testSame(self):
        """NVERIFY with unchanged attribute should return NFS4ERR_SAME

        Covered valid equivalence classes: 1, 2, 3, 4, 5, 6, 7, 9, 12
        """

        # Fetch sizes for all objects
        obj_sizes = self.ncl.lookup_all_objects_and_sizes()
        
        # For each type of object, do nverify with same filesize. 
        for (lookupops, objsize) in obj_sizes:
            self._nverify(lookupops,{FATTR4_SIZE:objsize},error=[NFS4ERR_SAME])

    #
    # Testcases covering invalid equivalence classes.
    #
    def testNoFh(self):
        """NVERIFY without (cfh) should return NFS4ERR_NOFILEHANDLE

        Covered invalid equivalence classes: 8
        """
        attrs = nfs4lib.dict2fattr({FATTR4_SIZE:17})
        nverifyop = self.ncl.nverify_op(attrs)
        res = self.ncl.do_ops([nverifyop])
        self.assert_status(res, [NFS4ERR_NOFILEHANDLE])

    def testWriteOnlyAttributes(self):
        """NVERIFY with FATTR4_*_SET should return NFS4ERR_INVAL

        Covered invalid equivalence classes: 10

        Comments: See GetattrSuite.testWriteOnlyAttributes. 
        """
        lookupops = self.ncl.lookup_path(self.regfile)
        settime = settime4(set_it=SET_TO_SERVER_TIME4)
        for attr in [FATTR4_TIME_ACCESS_SET, FATTR4_TIME_MODIFY_SET]:
            self._nverify(lookupops,{attr:settime},
                          error=[NFS4ERR_INVAL])

    def testNonUTF8(self):
        """NVERIFY with non-UTF8 FATTR4_OWNER should return NFS4ERR_INVAL

        Covered invalid equivalence classes: 13
        """
        # FRED - NFS4_OK seems reasonable
        lookupops = self.ncl.lookup_path(self.regfile)
        for name in self.get_invalid_utf8strings():
            self._nverify(lookupops,{FATTR4_OWNER:name},
                          error=[NFS4ERR_INVAL, NFS4_OK])

    def testRdattr_error(self):
        """NVERIFY with FATTR4_RDATTR_ERROR set should return NFS4ERR_INVAL

        Covered invalid equivalence classes: 10

        Comments: See GetattrSuite.testWriteOnlyAttributes. 
        """
        lookupops = self.ncl.lookup_path(self.regfile)
        self._nverify(lookupops,{FATTR4_RDATTR_ERROR:NFS4ERR_OP_ILLEGAL},
                     error=[NFS4ERR_INVAL])

    def testUnsupported(self):
        """NVERIFY with an unsupported attribute returns NFS4ERR_ATTRNOTSUPP
        """
        
        lookupops = self.ncl.lookup_path(self.regfile)
        for (bitnum,value,name) in self.unsupportedAttrs(self.regfile):
            self._nverify(lookupops,{bitnum:value},error=[NFS4ERR_ATTRNOTSUPP])
